﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Controls;
using Telerik.Windows.Controls.Map;
using System.Windows.Threading;
using System.Collections.ObjectModel;
using System.Threading;

namespace DynamicLayerPerformance
{
    public partial class MainPage : UserControl
    {
        public List<object> shapes = new List<object>();

        public MainPage()
        {
            InitializeComponent();

            Random rnd = new Random();
            for (int i = 0; i < 100000; i++)
            {
                Location location = new Location(rnd.NextDouble() * 160 - 80, rnd.NextDouble() * 300 - 150);
                location.Description = i.ToString();
                shapes.Add(location);
            }

            this.InitializeDynamicSource();
        }

        private void InitializeDynamicSource()
        {
            const int viewportWidth = 512;
			const int viewportHeight = 512;

            for (int zoomLayerIndex = 1; zoomLayerIndex <= 20; zoomLayerIndex += 1)
            {
                this.dynamicLayer.ZoomGridList.Add(new ZoomGrid((int)(Math.Pow(2, zoomLayerIndex + 8) / viewportWidth),
                    (int)(Math.Pow(2, zoomLayerIndex + 8) / viewportHeight), zoomLayerIndex));
            }

            this.dynamicLayer.DynamicSource = new DynamicSource(this);
        }

        private class DynamicSource : IMapDynamicSource
        {
            private MainPage mainPage;

            public DynamicSource(MainPage mainPage)
            {
                this.mainPage = mainPage;
            }

            public void ItemsRequest(object sender, ItemsRequestEventArgs e)
            {
                var shapes = from shp
                                 in mainPage.shapes
                             where ((Location)shp).Latitude <= e.UpperLeft.Latitude && ((Location)shp).Latitude >= e.LowerRight.Latitude &&
                                 ((Location)shp).Longitude >= e.UpperLeft.Longitude && ((Location)shp).Longitude <= e.LowerRight.Longitude
                                 select shp;

                e.CompleteItemsRequest(shapes);
            }
        }
    }
}
